from adafruit_ht16k33.matrix import Matrix8x8x2  # type: ignore
import alsaaudio as aa  # type: ignore
import board  # type: ignore
import busio  # type: ignore
import numpy as np
from struct import unpack
import wave


i2c = busio.I2C(board.SCL, board.SDA)  # Create the I2C interface

display = Matrix8x8x2(i2c)  # Create bi-color LED matrix
display.auto_write = False  # Disable auto write to improve performance

spectrum = [display.LED_GREEN,
            display.LED_GREEN,
            display.LED_GREEN,
            display.LED_YELLOW,
            display.LED_YELLOW,
            display.LED_YELLOW,
            display.LED_RED,
            display.LED_RED]
matrix = [0, 0, 0, 0, 0, 0, 0, 0]
power = []
weighting = [2, 8, 8, 16, 16, 32, 32, 64]

# Audio setup
wavfile = wave.open('/home/pi/Music/Secret Agent.wav', 'r')
sample_rate = wavfile.getframerate()
no_channels = wavfile.getnchannels()
chunk = 4096  # Use a multiple of 8

# ALSA
output = aa.PCM(aa.PCM_PLAYBACK, aa.PCM_NORMAL)
output.setchannels(no_channels)
output.setrate(sample_rate)
output.setformat(aa.PCM_FORMAT_S16_LE)
output.setperiodsize(chunk)


# Return power array index corresponding to a particular frequency
def piff(val):
    return int(2 * chunk * val / sample_rate)


def calculate_levels(data, chunk, sample_rate):
    global matrix

    # Convert raw data (ASCII string) to numpy array
    data = unpack("%dh" % (len(data)/2), data)
    data = np.array(data, dtype='h')

    # Apply FFT - real data
    fourier = np.fft.rfft(data)
    # Remove last element in array to make it the same size as chunk
    fourier = np.delete(fourier, len(fourier)-1)
    # Find average 'amplitude' for specific frequency ranges in Hz
    power = np.abs(fourier)
    matrix[0] = int(np.mean(power[piff(0):piff(156):1]))
    matrix[1] = int(np.mean(power[piff(156):piff(313):1]))
    matrix[2] = int(np.mean(power[piff(313):piff(625):1]))
    matrix[3] = int(np.mean(power[piff(625):piff(1250):1]))
    matrix[4] = int(np.mean(power[piff(1250):piff(2500):1]))
    matrix[5] = int(np.mean(power[piff(2500):piff(5000):1]))
    matrix[6] = int(np.mean(power[piff(5000):piff(10000):1]))
    matrix[7] = int(np.mean(power[piff(10000):piff(20000):1]))

    # Tidy up column values for the LED matrix
    matrix = np.divide(np.multiply(matrix, weighting), 1000000)
    # Set floor at 0 and ceiling at 8 for LED matrix
    matrix = matrix.clip(0, 8)
    return matrix


# Start reading .wav file
data = wavfile.readframes(chunk)
# Loop while audio data present
while data != b'':
    output.write(data)
    try:
        matrix = calculate_levels(data, chunk, sample_rate)
        display.fill(display.LED_OFF)  # Clear matrix
        for y in range(0, 8):
            for x in range(0, int(matrix[y])):
                display[x, y] = spectrum[x]
        display.show()  # Illuminate LEDs
    except Exception as e:
        print(f"An error occurred: {e}")

    data = wavfile.readframes(chunk)

print("Done.")
